//  $Id: DLLFN.H 1.16 2003/03/10 18:19:52 thomd Exp $
//
//  *** DO NOT REMOVE THE FOLLOWING NOTICE ***
//
//  Copyright (c) 1998-2003 Datalight, Inc.
//  All Rights Reserved
//
//  Datalight, Inc. is a Washington corporation located at:
//
//  21520 30th Dr SE, Suite 110,          Tel: +1.425.951.8086
//  Bothell, WA  98021                    Fax: +1.425.951.8094
//  USA                                   Http://www.datalight.com
//
//  This software, including without limitation all source code and
//  documentation, is the confidential, trade secret property of
//  Datalight, Inc., and is protected under the copyright laws of
//  the United States and other jurisdictions.  Portions of the
//  software may also be subject to one or more the following US
//  patents: US#5860082, US#6260156.
//
//  In addition to civil penalties for infringement of copyright
//  under applicable U.S. law, 17 U.S.C. 1204 provides criminal
//  penalties for violation of (a) the restrictions on circumvention
//  of copyright protection systems found in 17 U.S.C. 1201 and
//  (b) the protections for the integrity of copyright management
//  information found in 17 U.S.C. 1202.
//
//  U.S. Government Restricted Rights.  Use, duplication,
//  reproduction, or transfer of this commercial product and
//  accompanying documentation is restricted in accordance with
//  FAR 12.212 and DFARS 227.7202 and by a License Agreement.
//
//  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS
//  PROTECTED UNDER A SOURCE CODE AGREEMENT, NON-DISCLOSURE
//  AGREEMENT (NDA), OR SIMILAR BINDING CONTRACT BETWEEN DATALIGHT,
//  INC. AND THE LICENSEE ("BINDING AGREEMENT").  YOUR RIGHT, IF ANY,
//  TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,IS SUBJECT
//  TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT, AND BY
//  USING THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE
//  TO BE BOUND BY THE TERMS OF THE BINDING AGREEMENT.  CONTACT
//  DATALIGHT, INC. AT THE ADDRESS SET FORTH ABOVE IF YOU OBTAINED
//  THIS SOFTWARE IN ERROR.
//
/********************************************************************
	$DOCMODULE:DLLFN.H

	Copyright (c) 1998-2002 Datalight, Inc.
   All Rights Reserved.
   Datalight Confidential and Proprietary

	Structures and prototypes for Long Filename functions. Most of
	these functions are just wrappers around the Int 21h longname
	calls.

	$DOCHISTORY
	03/01/1999 dg  Added more Smart functions
	02/16/1999 dg  Added SmartXxx() functions for COMMAND and utils.
	07/28/1998 de  Added an already included guard, and declared the
                  LFNPresent() function.
	06/29/1998 dg  Created by Drew Gislason.
*********************************************************************/

#if !defined( DLLFN_H )
#define 		  DLLFN_H 1

// dldoserr.asm: return (and reset) record of last DOS I/O error code
//
int PASCAL dlCheckDOSError(void);

// note: this macro ONLY works after a call to Int 21h 71xxh. It
// checks the return error code.
//
//  This was removed becuase it was used to check calls which caused
//  MSDOS do hang.  You can not use the back door/server call with
//  invalid function numbers and then check to see if it was an invalid
//  function.  You should use regular function 71xxh to determine
//  if lfn functions are available.  Perhaps 7143h provides minimal
//  overhead.
//
//#define LFNsInstalled() (dlCheckDOSError() != ERR_INVALID_FUNCTION)

/* general string lengths */
#define LFN_MAXNAME 255		/* up to 255 characters (plus null) */
#define DOTNAMESIZE 14		/* up to 14 characters in an 8.3 filename */

// "generic" maximum path. may in the future be short or long depending
// on compile mode.  LFN_MAX_PATH is from DOSDEF.H
#define DL_MAXPATH	LFN_MAX_PATH

/* DOS File Date on disk */
typedef struct
{
	unsigned day : 5;		/* 1-31 */
	unsigned month : 4;	/* 1-12 */
	unsigned year : 7;	/* 1980+year, 1980-2107 */
} FileDate;

#define FileYear(y) ((y)-1980)

/* DOS File Time on disk */
typedef struct
{
	unsigned second : 5;		/* second/2 0-29 */
	unsigned minute : 6;		/* 0-59 */
	unsigned hour : 5;		/* 0-23 */
} FileTime;


/* helper function for date/time structures */
typedef unsigned word;
#define WORD(n) (*(word *)(&(n)))

/* standard file attributes */
#define A_NORMAL		0x00
#define A_READONLY	0x01
#define A_HIDDEN		0x02
#define A_SYSTEM		0x04
#define A_VOLUME		0x08
#define A_SUBDIR		0x10
#define A_ARCHIVE		0x20
#define A_ALLDIR		0x17

/* for those functions that require attributes */
#define MUST_MATCH_ATTR(a) ((int)(a)<<8)
#define SEARCH_ATTR(a) ((int)(a)&0xFF)

/* delete options */
#define DEL_NOWILD	0
#define DEL_WILD		1

/* iModeFlags for LFNCreateOpenFile() */
#define OPEN_ACCESS_READONLY				0
#define OPEN_ACCESS_WRITEONLY				1
#define OPEN_ACCESS_READWRITE				2
#define OPEN_ACCESS_RO_NOMODLASTACCESS	4
#define OPEN_SHARE_COMPATIBLE				0
#define OPEN_SHARE_DENYREADWRITE		0x10
#define OPEN_SHARE_DENYWRITE			0x20
#define OPEN_SHARE_DENYREAD			0x30
#define OPEN_SHARE_DENYNONE			0x40
#define OPEN_FLAGS_NOINHERIT			0x80
#define OPEN_FLAGS_NO_BUFFERING		0x100
#define OPEN_FLAGS_NO_COMPRESS		0x200
#define OPEN_FLAGS_ALIAS_HINT			0x400
#define OPEN_FLAGS_NOCRITERR			0x2000
#define OPEN_FLAGS_COMMIT				0x4000

/* actions for LFNCreateOpenFile() */
#define FILE_CREATE	0x10
#define FILE_OPEN			1
#define FILE_TRUNCATE	2

/* actions taken by LFNCreateOpenFile() */
#define ACTION_OPENED				1
#define ACTION_CREATED_OPENED		2
#define ACTION_REPLACED_OPENED	3

/* actions for LFNExtendedGetSetAttr() */
#define EXT_ACT_GET_ATTR							0
#define EXT_ACT_SET_ATTR							1
#define EXT_ACT_GET_PHYSICAL_SIZE				2
#define EXT_ACT_SET_LAST_WRITE_DATE_TIME		3
#define EXT_ACT_GET_LAST_WRITE_DATE_TIME		4
#define EXT_ACT_SET_LAST_ACCESS_DATE			5
#define EXT_ACT_GET_LAST_ACCESS_DATE			6
#define EXT_ACT_SET_CREATION_DATE_TIME			7
#define EXT_ACT_GET_CREATION_DATE_TIME			8

/* findfirst/next: date time formats */
#define WIN32DATE 0
#define DOSDATE   1

/* 64-bit file date/time expressed in 100-nanoseconds since 1-1-1901 */
struct DATETIME64
{
	ulong lowDateTime;
	ulong highDateTime;
};

/* date/time using DOS file date/time */
struct DATETIMEDOS
{
	FileTime time;
	FileDate	date;
};

/* the union of the two */
union FILETIME
{
	struct DATETIME64 nano;
	struct DATETIMEDOS dos;
};

/* returned by findfirst/next */
struct WIN32_FIND_DATA
{
   uchar	attr;								// 00h file attributes
   uchar	upperAttr[3];					// 01h not used (upper attribs)
   union FILETIME creationTime;		// 04h only with long filenames
   union FILETIME lastAccessDate;	// 0Ch only with long filenames
   union FILETIME lastWriteTime;		// 14h standard DOS file time/date
	ulong	fileSizeHigh;					// 1Ch unused by FAT file system
   ulong	fileSize;						// 20h file size
	ulong reserved[2];					// 24h ???
	char	szFileName[LFN_MAX_PATH];	// 2Ch long filename, or 8.3 if no lfn
	char	szAliasFileName[DOTNAMESIZE];	// 12Ch 8.3 alias
};

/* findfirst/next: all characters converted from UniCode OK? */
#define CONVERT_OK			0
#define CONVERT_PRIMARY		1
#define CONVERT_SECONDARY	2

/* actions used by LFNGetFullPath() */
#define FULLPATH_DEFNAME		(0 | FULLPATH_SUBST)
#define FULLPATH_SHORTNAME		(1 | FULLPATH_SUBST)
#define FULLPATH_LONGNAME		(2 | FULLPATH_SUBST)

/* actions used by LFNGetFullPath() not showing substed or network drives */
#define _FULLPATH_DEFNAME		0
#define _FULLPATH_SHORTNAME	1
#define _FULLPATH_LONGNAME		2

/* subst flags for LFNGetFullPath() */
#define FULLPATH_NOSUBST		0
#define FULLPATH_SUBST			0x8000

/* returned by 71A6h Get Longname File Info */
struct BY_HANDLE_FILE_INFORMATION
{
   uchar	attr;								// 00h file attributes
   uchar	upperAttr[3];					// 01h not used (upper attribs)
   union FILETIME creationTime;		// 04h only with long filenames
   union FILETIME lastAccessDate;	// 0Ch only with long filenames
   union FILETIME lastWriteTime;		// 14h standard DOS file time/date
   ulong volumeSerialNumber;			// 1Ch serial # of disk (if any)
   ulong fileSizeHigh;					// 20h
   ulong	fileSize;						// 24h file size
   ulong numLinks;						// 28h # of times file is opened
   ulong fileIndexHigh;					// 2Ch
   ulong fileIndex;						// 30h index to file
};

/* actions used by LFNConvertFileTime() */
#define NANOTIME_TO_DOS		0
#define DOSTIME_TO_NANO		1

/* charsets for LFNGenerateShortName() */
#define BCS_WANSI		0x00
#define BCS_OEM		0x11
#define BCS_UNICODE	0x22

/* iNameType shortname formats for LFNGenerateShortName() */
#define SNFMT_ENAME	0		/* expand to 11 chars exactly */
#define SNFMT_DOT		1		/* standard 8.3 asciiz form */

// used for Volume Info
#define FS_CASE_SENSITIVE		0x0001
#define FS_CASE_IS_PRESERVED	0x0002
#define FS_UNICODE_ON_DISK		0x0004
#define FS_LFN_APIS				0x4000
#define FS_VOLUME_COMPRESSED	0x8000
#define FS_FLAGS (FS_CASE_IS_PRESERVED | FS_UNICODE_ON_DISK | FS_LFN_APIS)

/* LFNDATE.ASM */
bool PASCAL LFNGetLastAccessDate(int nHandle, FileDate *pLastAccessDate);
bool PASCAL LFNSetLastAccessDate(int nHandle, word lastAccessDate);
bool PASCAL LFNGetCreateTimeDate(int nHandle, FileTime *pCreateTime,
									FileDate *pCreateDate, int *pCreateMilli);
bool PASCAL LFNSetCreateTimeDate(int nHandle, word createTime,
									word createDate, int createMilli);

/* LFNDIR.ASM */
bool PASCAL LFNMakeDirectory(char szLongName[]);
bool PASCAL LFNRemoveDirectory(char szLongName[]);
bool PASCAL LFNChangeDirectory(char szLongName[]);
bool PASCAL LFNGetCurrentDirectory(int nDrive, char szLongName[]);

/* LFNCREAT.ASM */
bool PASCAL LFNCreateOpenFile(int iModeFlags, int iAttr, int iAction,
					char szLongName[], int iAliasHint,
					int *pHandle, int *pActionTaken);
bool PASCAL LFNDeleteFiles(int iAttrs, char szLongMask[], bool bWildOK);

/* LFNATTR.ASM */
bool PASCAL LFNExtendedGetSetAttr(int iAction, FileTime *pTime, unsigned *pAttr,
					char szLongName[], FileDate *pDate, int *pMilli, ulong *pFileSize);

/* LFNFIND.ASM */
bool PASCAL LFNFindFirstFile(int nAttrs, char szLongMask[], int nDateFmt,
			struct WIN32_FIND_DATA *pFindData, int *pHandle, int *pConvertCode);
bool PASCAL LFNFindNextFile(int nHandle, int nDateFmt,
			struct WIN32_FIND_DATA *pFindData, int *pConvertCode);
bool PASCAL LFNFindClose(int nHandle);

/* LFNREN.ASM */
bool PASCAL LFNRenameFile(char szOldLongName[], char szNewLongName[]);

//  There is not a version of this file yet.  It needs to be cut out
//  of command and placed in a smart function only becuase 7156h does
//  not support wild cards.  Because you have to support LFNs too, you
//  will need all the unicode functions for name and path munipulations
//  as well.  For the sake of time, it was decided to fix it in command
//  which already had the functions rather than making this and all the
//  required functions autonimus.
//
//bool PASCAL LFNWildcardRename(char szOldLongName[], char szNewLongName[]);

/* LFNPATH.ASM */
bool PASCAL LFNGetFullPath(int iFlags, char szSrcLongName[],
					char szDstLongName[]);

/* LFNINFO.ASM */
bool PASCAL LFNGetVolumeInformation(int iBufSize, char szRootName[],
					char szFileSystemName[],
					int *pFlags, int *pMaxName, int *pMaxPath);
bool PASCAL LFNGetFileInfoByHandle(int nHandle,
					struct BY_HANDLE_FILE_INFORMATION *pFileInfo);

/* LFNMISC.ASM */
bool PASCAL LFNConvertFileTime(int iAction, int *pMilli,
												union FILETIME *pFileTime);
bool PASCAL LFNGenerateShortName(int iCharSet, int iNameType,
									char szLongName[], char szShortName[]);

/* LFNSUBST.ASM */
//
// These iFunction definitions are used by the multiplex LFN subst function
//
#define LFN_CREATE_SUBST      0
#define LFN_TERMINATE_SUBST   1
#define LFN_QUERY_SUBST       2

bool PASCAL LFNSubstFunction(int iFunction, int iDrive, char szLongName[]);


//  netlfn.c
//
//  Determine if the specified drive supports LFN calls.  DOS networks
//  don't but this function allows for exception cases to be handled in
//  one place.
//
//  ucDrive = (0=Default,1=A,2=B,...)
//
bool DriveSupportsLFNs( uchar ucDrive );

// smtxpand.c
//
bool PASCAL SmartExpandPath( int iLFNExpandFlags, char * szPath,
		char * szExpandedPath );

// lfnsmart.c
//
bool PASCAL LFNPresent(void);
bool PASCAL SmartGetCurrentDirectory(int nDrive, char szPathName[]);
bool PASCAL SmartChangeDirectory(char szPathName[]);
bool PASCAL SmartMakeDirectory(char szPathName[]);
bool PASCAL SmartRemoveDirectory(char szPathName[]);
bool PASCAL SmartCreateOpenFile(int iModeFlags, int iAttr, int iAction,
					char szFileName[], int *pHandle, int *pActionTaken);
//
//  There is not a version of this file yet.  It needs to be cut out
//  of command and placed in a smart function only becuase 7156h does
//  not support wild cards.  Because you have to support LFNs too, you
//  will need all the unicode functions for name and path munipulations
//  as well.  For the sake of time, it was decided to fix it in command
//  which already had the functions rather than making this and all the
//  required functions autonimus.
//
//bool PASCAL SmartWildcardRename(char * szOldName, char * szNewName);
//
bool PASCAL SmartFindFirst(int nAttrs, char szPathMask[],
					PFIND pFindData);
bool PASCAL SmartFindNext( PFIND pFindData);
bool PASCAL SmartFindClose( PFIND pFindData);

bool PASCAL SmartFindAreAllClosed(void);
void PASCAL SmartFindCloseAll(void);

//  Return the address of the LFN in the GLOBAL LFN find
//  structure.  Be advised that this variable goes stale
//  with subsequent find first/next calls.  This was
//  placed here as a temporary optimization to for
//  retrieving the long filename from our NOT so smart
//  find first/next functions.
//
void PASCAL GetSmartFindLFNAddress( char ** pszLFN );


// WILDDEL.C
//
bool PASCAL SmartWildcardDelete(char * szFileName);

// SMTDEL.C
//
bool PASCAL	SmartDelete(char * szFileName);

// SMTREN.C
//
bool PASCAL SmartRenameFileOrDirectory(char * szOldName, char * szNewName);

//  requires the "special" ffblk (see dostruct.h FIND structure)
//  returned from SmartFindFirst().
//  returns 0 if normal ffblk.
//
unsigned PASCAL SmartGetLastAccessDate( LPFIND lpFindData);


// lfnsplit.c
//
int PASCAL LFNSplitFileName(char *szSrcName, char *szDrive,
				char *szDirectory, char *szFileName);

// buffer sizes required by InternationalSplitFileName()
// szDirectory and szFileName must be DL_MAXPATH in size!
#define SPLIT_MAXDRIVE	3		// maximum size of szDrive

// flags returned by InternationalSplitFileName()
#define SPLIT_WILDCARDS 0x01
#define SPLIT_FILENAME  0x04
#define SPLIT_DIRECTORY 0x08
#define SPLIT_DRIVE     0x10

// get drive freespace, whether FAT12, FAT16 or FAT32 drive.
bool PASCAL SmartGetDriveFreeSpace(int drive,
				ulong *pulSectorsPerCluster,
				ulong *pulFreeClusters,
				ulong *pulBytesPerSector,
				ulong *pulTotalClusters);

// get file attributes
bool PASCAL SmartGetFileAttributes(char *pName, unsigned *pAttributes);
bool PASCAL SmartSetFileAttributes(char *szName, unsigned uAttributes);

// longname argument manipulation
char * PASCAL LFNNextArg(char *szArg);
char * PASCAL LFNEndArg(char *szArg);
char * PASCAL LFNSkipWhite(char *szArg);
void PASCAL LFNStripArgQuotes(char *szArg);

bool PASCAL ComputeENAMEChecksum( char far * szENAME,
		uchar far * pucChecksum );


//  Constants for dlGenShortFilename (DOS fn 71a8 - Generate short name)
//  output short name format.
//
#define GEN_SHORT_NAME				0
#define GEN_SHORT_ENAME       	1

//  Constants for dlGenShortFilename (DOS fn 71a8 - Generate short name)
//  input long name format
//
#define GEN_SHORT_NAME_ANSI		0
#define GEN_SHORT_NAME_OEM			1
#define GEN_SHORT_NAME_UNICODE	2


//  i21f71a8.asm DOS fn 71a8 - Generate short name
//
//  dlGenShortFilename
//
//  This module is a wrapper for interrupt 21h function 71A8h which
//  generates a short filename from the specified long filename.
//
//  On Entry
//
//    szLongFilename  = The long filename to be converted
//    szShortFilename = 13 byte work buffer (26 bytes for Unicode)
//    ucReturnFormat  = 00h 11-char directory entry/FCB filename format
//		                  01h DOS 8.3
//								<<SEE CONSTANTS ABOVE>>
//    ucCharSetMask   = bits 7-4: short name's character set
//		                  bits 3-0: long name's character set
//		                    00h Windows ANSI
//			                 01h current OEM character set
//			                 02h Unicode
//								  <<SEE CONSTANTS ABOVE>>
//
//  On Exit
//
//    if the function is successful, the szShortFilename will contain
//    the short filename with respect to the long filename.
//
//	   ax  TRUE if function is successful
//
bool PASCAL dlGenShortFilename( char * szLongFilename,
		uchar ucReturnFormat, uchar ucCharSetMask, char * szShortFilename );

#endif  // #if !defined( DLLFN_H )
