//  $Id: MEMEXT.C 1.6 2003/03/20 16:57:44 thomd Exp $
//
//  *** DO NOT REMOVE THE FOLLOWING NOTICE ***
//
//  Copyright (c) 1991-2003 Datalight, Inc.
//  All Rights Reserved
//
//  Datalight, Inc. is a Washington corporation located at:
//
//  21520 30th Dr SE, Suite 110,          Tel: +1.425.951.8086
//  Bothell, WA  98021                    Fax: +1.425.951.8094
//  USA                                   Http://www.datalight.com
//
//  This software, including without limitation all source code and
//  documentation, is the confidential, trade secret property of
//  Datalight, Inc., and is protected under the copyright laws of
//  the United States and other jurisdictions.  Portions of the
//  software may also be subject to one or more the following US
//  patents: US#5860082, US#6260156.
//
//  In addition to civil penalties for infringement of copyright
//  under applicable U.S. law, 17 U.S.C. 1204 provides criminal
//  penalties for violation of (a) the restrictions on circumvention
//  of copyright protection systems found in 17 U.S.C. 1201 and
//  (b) the protections for the integrity of copyright management
//  information found in 17 U.S.C. 1202.
//
//  U.S. Government Restricted Rights.  Use, duplication,
//  reproduction, or transfer of this commercial product and
//  accompanying documentation is restricted in accordance with
//  FAR 12.212 and DFARS 227.7202 and by a License Agreement.
//
//  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS
//  PROTECTED UNDER A SOURCE CODE AGREEMENT, NON-DISCLOSURE
//  AGREEMENT (NDA), OR SIMILAR BINDING CONTRACT BETWEEN DATALIGHT,
//  INC. AND THE LICENSEE ("BINDING AGREEMENT").  YOUR RIGHT, IF ANY,
//  TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,IS SUBJECT
//  TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT, AND BY
//  USING THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE
//  TO BE BOUND BY THE TERMS OF THE BINDING AGREEMENT.  CONTACT
//  DATALIGHT, INC. AT THE ADDRESS SET FORTH ABOVE IF YOU OBTAINED
//  THIS SOFTWARE IN ERROR.
//
/********************************************************************
	$DOCMODULE:MEMEXT.C

	Copyright (c) 1991-2002 Datalight, Inc.
   All Rights Reserved.
   Datalight Confidential and Proprietary.

	Fixed location extended memory RAM disk.

	Syntax:
	DEVICE=MEMEXT.SYS [DiskSize [SectorSize [NumEntries]]] [/L#] [/S]

	/L#   Linear address for start of disk (example 1000000 is 16 Megs)
	/S    Static: if disk already exists, don't reinitialize

	DiskSize		Size of disk in Kilobytes. Valid range is 4 - 32768.
	SectorSize	Size of each disk sector. Valid values are 128, 256 and 512.
	NumEntries	Number of root directory entries.

	This extended memory disk is just like a VDISK, but resides at a
	fixed location, rather than the next available location. Does NOT
	use XMS or EMS memory. HIMEM and EMM386 are incompatible with this
	driver.

	$DOCHISTORY
	07/23/1998 dg  Updated to Solid Code.
	08/12/1991 dg  Created by Drew Gislason.
*********************************************************************/
#define __FILENUM__	203
#include "dosver.h"
#include "compiler.h"
#include "memdisk.h"
#include "vdisk.h"

//#include "fatworld.h"
bool PASCAL dlIsFat32World( void );

/* cmdline options and arguments */
bool	bStatic;								/* /S option  */

/* disk parameters */
unsigned nTotalSecs;
unsigned nDiskSizeK = DEF_SIZEK;
int	nBytesPerSector = DEF_SEC_SIZE;
int	nRootEntries = DEF_DIRS;

#define DEF_DIR_ENTRIES 	DEF_DIRS
struct BPB std_bpb =
{
	DEF_SEC_SIZE,1,1,2,DEF_DIR_ENTRIES,DEF_SIZEK*2,FAT_ID,6,9,2,0
};
char std_volume[] = "VDISK V6.22";
unsigned long std_serial = 0x12345678L;
char std_fattype[] = "FAT12    ";

/* start of disk expressed as a linear address */
ulong	lStartAddr;

/* messages */
char szSignOn[]    = "\nMEMEXT %s %s\n%s\n\n";
char szErrFormat[] = "Format failed!";
char szErrBadOpt[] = "Invalid option \"/%s\"!\n";
char szErrBadArgs[]= "Too many arguments!\n";
char szInstalled[] = "Installed %dKB Extended %sRAM disk as drive %c:\n";
char szStatic[]    = "(static) ";

/* Default 1Meg extended disk at 4Meg boundary */
#if BUILTIN
char szBuiltinCmdline[] = "1024 /L400000";
#endif

bool fIsFat32World;

void DisplaySignOnMessage( void )
{
	if( fIsFat32World )
	{
		printf(szSignOn,SHORTVERSION,BUILDVERSION,COPYRIGHT);
	}
	else
	{
		printf(szSignOn,SHORTVERSION622,BUILDVERSION,COPYRIGHT);
	}
}

/*
	$DOCPROC:meminit

	Initialize the Exended RAM disk.

	On Entry
		cmdline	the cmdline from device= in CONFIG.SYS
		drv		drive # assigned from DOS (0=A, 1=B, etc..)
	On Exit
		bpb		updated with accurate BPB for RAM disk

	Returns 1 if worked, 0 if failed.
*/
#ifdef __TURBOC__
#pragma argsused
#endif
int meminit(struct BPB bpb[], char far *cmdline, unsigned *endseg, int drv)
{
	int		nSecPerKB;

	//  Determine if we are operating in a DOS 7.1 enviornment
	//
	fIsFat32World = dlIsFat32World();

	/* display sign-on */
	DisplaySignOnMessage();

	/* validate parameters */
	ASSERT(bpb);

	/* determine switches, and sizes suggested by user */
	#if BUILTIN
		cmdline = szBuiltinCmdline;
	#endif
	if(!ParseCmdline(cmdline))
		return 0;

	/* limit 8Meg for 128-byte sectors, 16Meg for 256 byte sectors */
	if(nDiskSizeK >= 8*1024 && nBytesPerSector == 128)
		nBytesPerSector = 256;
	if(nDiskSizeK >= 16*1024 && nBytesPerSector == 256)
		nBytesPerSector = 512;

	/* set total # of sectors */
	nSecPerKB = 1024 / nBytesPerSector;
	nTotalSecs = nDiskSizeK * nSecPerKB;

	/* actually format the media */
	if(!FormatRAMDisk(nBytesPerSector, nRootEntries, nTotalSecs))
	{
		printf(szErrFormat);
		return 0;
	}

	/* show installed message */
	printf(szInstalled, nDiskSizeK, (bStatic ? szStatic : ""), drv+'A');

	/* tell memdisk about this BPB */
	GetDiskBPB(bpb);

	/* one disk */
	return 1;
}

/*
	$DOCPROC:memread

   Read one or more sectors from the RAM disk into a buffer specified
	by DOS.

	On Entry
	   offset   The offset (in bytes) into the disk where the read
					starts.
	   len      The length of the read in bytes. The length cannot
					exceed 64KB - 1 sector.
	   buffer   Pointer to the buffer to read into.

	On Exit
	   Returns TRUE(non-zero) if worked, FALSE(zero) if failed.
*/
bool memread(long offset, unsigned len, char far *buffer)
{
	#if DEBUG > 1
		printf("memread(%lx, %x, %lp)\n",offset,len,buffer);
	#endif

	/* validate parameters */
	ASSERT(len);
	ASSERT(buffer);
	ASSERT(offset < 0x2000000L);	/* disks are always <= 32Meg */
	ASSERT(lStartAddr);

	/* read from extended memory */
	if(!CopyFromInt15(buffer, lStartAddr+offset, len/2))
		return FALSE;

	/* read worked */
	return TRUE;
}


/*
	$DOCPROC:memwrite

   Write one or more sectors to the RAM disk from a buffer specified
	by DOS.

	On Entry
	   offset   The offset (in bytes) into the disk where the write
					starts.
	   len      The length of the write in bytes. The length cannot
					exceed 64KB - 1 sector.
	   buffer   Pointer to the buffer to write from.

	On Exit
	   Returns TRUE(non-zero) if worked, FALSE(zero) if failed.
*/
bool memwrite(long offset, unsigned len, char far *buffer)
{
	/* display write address in DEBUG mode */
	#if DEBUG > 1
		printf("memwrite(%lx, %x, %lp)\n",offset,len,buffer);
	#endif

	/* validate parameters */
	ASSERT(len);
	ASSERT(buffer);
	ASSERT(lStartAddr);

	/* write to the extended memory */
	if(!CopyToInt15(lStartAddr+offset, buffer, len/2))
		return FALSE;

	/* write worked */
	return TRUE;
}

/*
	$DOCPROC:ParseCmdline

	Parse the command line arguments. Parses both switches and
	arguments as follows:

	Syntax:
	DEVICE=MEMEXT.SYS [DiskSize [SectorSize [NumEntries]]] [/L#] [/S]
                    ^
*/
bool ParseCmdline(char far *lpCmdline)
{
	char	c;
	int	nArgCount = 0;

	/* validate parameter */
	ASSERT(lpCmdline);

	/* look for command line arguments */
	while(1)
	{
		/* skip whitespace to next switch */
		lpCmdline = MemSkipWhite(lpCmdline);
		c = *lpCmdline;

		/* at end of line? */
		if(c < ' ')
			break;

		/* cmdline options (switches) */
		if(c == '/')
		{
			/* get switch char */
			++lpCmdline;					/* skip '/' */
			c = MemToUpper(*lpCmdline++);

			switch(c)
			{
				/* determine address for */
				case 'L':
					lStartAddr = 0;
					while(MemIsXDigit(*lpCmdline))
					{
						lStartAddr = _lmulu(lStartAddr, 0x10) +
														MemXDigitVal(*lpCmdline);
						++lpCmdline;
					}
					break;

				/* RAM disk should remain static, if possible */
				case 'S':
					bStatic = TRUE;
					break;

				default:
					printf(szErrBadOpt, c);
					return FALSE;
			}
		}

		/* must be a cmdline argument */
		else
		{
			switch(nArgCount)
			{
				/* arg0 is disk sizeK */
				case 0:
					nDiskSizeK = MinMax(lpCmdline, 4, 32767);
					break;

				/* arg1 is bytes per sector */
				case 1:
					nBytesPerSector = MinMax(lpCmdline, 128, 512);
					if(nBytesPerSector != 128 && nBytesPerSector != 256)
						nBytesPerSector = 512;
					break;

				/* arg2 is # root directory entires */
				case 2:
					nRootEntries = MinMax(lpCmdline, 4, 512);
					break;

				/* too many arguments */
				case 3:
					printf(szErrBadArgs);
					return FALSE;
			}

			/* skip the argument */
			lpCmdline = MemSkipArg(lpCmdline);

			/* one more argument */
			++nArgCount;
		}

	} /* end of while */

	/* everything worked */
	return TRUE;
}

#if DEBUG
/*
	$DOCPROC:AssertFail

	When an assertion fires, this function is called. Only defined if
	DEBUG is defined.
*/
void PASCAL AssertFail(int nFile, int nLine)
{
	panic("MEMEXT Internal Error %d.%d!\n", nFile, nLine);
}
#endif	/* DEBUG */


#if UNIT_TEST
/*
	Unit Test entry point.
*/
int main(void)
{
	static char szUnitTestCmdline[] = " 1024 /L400000";

	/* return error code if failed. messages are displayed by test. */
	if(!RamDiskUnitTest(szUnitTestCmdline))
		return 1;

	/* no errors */
	return 0;
}
#endif	/* UNIT_TEST */
