//  $Id: SC400PAG.C 1.6 2003/03/20 16:58:02 thomd Exp $
//
//  *** DO NOT REMOVE THE FOLLOWING NOTICE ***
//
//  Copyright (c) 1995-2003 Datalight, Inc.
//  All Rights Reserved
//
//  Datalight, Inc. is a Washington corporation located at:
//
//  21520 30th Dr SE, Suite 110,          Tel: +1.425.951.8086
//  Bothell, WA  98021                    Fax: +1.425.951.8094
//  USA                                   Http://www.datalight.com
//
//  This software, including without limitation all source code and
//  documentation, is the confidential, trade secret property of
//  Datalight, Inc., and is protected under the copyright laws of
//  the United States and other jurisdictions.  Portions of the
//  software may also be subject to one or more the following US
//  patents: US#5860082, US#6260156.
//
//  In addition to civil penalties for infringement of copyright
//  under applicable U.S. law, 17 U.S.C. 1204 provides criminal
//  penalties for violation of (a) the restrictions on circumvention
//  of copyright protection systems found in 17 U.S.C. 1201 and
//  (b) the protections for the integrity of copyright management
//  information found in 17 U.S.C. 1202.
//
//  U.S. Government Restricted Rights.  Use, duplication,
//  reproduction, or transfer of this commercial product and
//  accompanying documentation is restricted in accordance with
//  FAR 12.212 and DFARS 227.7202 and by a License Agreement.
//
//  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS
//  PROTECTED UNDER A SOURCE CODE AGREEMENT, NON-DISCLOSURE
//  AGREEMENT (NDA), OR SIMILAR BINDING CONTRACT BETWEEN DATALIGHT,
//  INC. AND THE LICENSEE ("BINDING AGREEMENT").  YOUR RIGHT, IF ANY,
//  TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,IS SUBJECT
//  TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT, AND BY
//  USING THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE
//  TO BE BOUND BY THE TERMS OF THE BINDING AGREEMENT.  CONTACT
//  DATALIGHT, INC. AT THE ADDRESS SET FORTH ABOVE IF YOU OBTAINED
//  THIS SOFTWARE IN ERROR.
//
/********************************************************************
	$DOCMODULE:SC400PAG.C

	Copyright (c) 1995-2002 Datalight, Inc.
   All Rights Reserved.
   Datalight Confidential and Proprietary

	This module is a paging mechanism for the AMD Elan SC400
	microcontroller. It is used both by the Elan SC400 Paged Disk
	Driver and the Elan SC400 Paged Loader.

	This mechanism uses the MMS Page 5 register, which puts the
	window at B000:0. This makes the paged disk incompatible with a
	monochrome video card.

   Modules:
   MEMDISK+MEMPAGED+SC400PAG   Makes the Elan SC400 Paged Disk Driver.
   LDRBEXT+PAGEDLDR+SC400PAG   Makes the Elan SC400 Paged Loader.

	$DOCHISTORY:
	07/23/1998 dg  Updated to Solid Code Standards
	06/23/1998 dg  Now compiles/works with MSVC 1.52
	11/20/1995 dg  Created by Drew Gislason
*******************************************************************/
#include "compiler.h"
#include "memdisk.h"

/* Size of the "window" into paged memory */
#define PAGESIZE (32L * 1024)

/* Segment & Offset of "window" into paged memory, B000:0 */
#define WINDOW_PTR 0xB0000000L

/* This string is used to identify the paged disk type at signon */
char szPagedDiskIDString[]={"(AMD ElanSC400)"};
long lPagedDiskWindowSize = PAGESIZE;
uchar far *lpPagedDiskWindow = (uchar far *)WINDOW_PTR;

/* ROMCS may be 0,1 or 2. 1 is recommended */
#define ROMCS		1

/* a useful type */
typedef unsigned int WORD;

/*
	$DOCPROC:set_page

	This set_page routine is designed for the AMD Elan SC400
	board (rev 1, 2 and 3), and uses the MMS Page 5 register
	to map the onboard Flash into ROMCS1

	MMS Page 5 Destination Register        I/O Address 22H/23H
	 Bits   R/W  Description	                      Index 32H
	+-----+-----+--------------------------------+
	| 7:0 | R/W | Bits 22:15 of start address    |
	+-----+-----+--------------------------------+

	MMS Page 5 Dest/Attr Register          I/O Address 22H/23H
	 Bits   R/W  Description	                      Index 33H
	+-----+-----+--------------------------------+
	| 2:0 | R/W | Bits 25:23 of start address    |
	+-----+-----+--------------------------------+
	| 3   | R/W | 1=Enable, 0=Disable MMS Page 5 |
	+-----+-----+--------------------------------+
	| 4   | R/W | 1=Cacheable, 0=No Cache        |
	+-----+-----+--------------------------------+
	| 5   | R/W | 1=Write Protect, 0=Writable    |
	+-----+-----+--------------------------------+
	| 7:6 | R/W | Bits 22:15 of start address    |
	+-----+-----+--------------------------------+

	The above indexed registers set up a 32K window at B000:0
	that points to the memory, on 32K boundaries.

	Returns TRUE if worked, FALSE if failed.
*/
bool set_page(unsigned nPage)
{
	unsigned long lAddress;

	/* determine linear address in the 4 Meg of Flash */
	lAddress = _lmulu(PAGESIZE, nPage);

	#if DEBUG
		printf("set_page(%d), lAddress %lx CS%d",nPage, lAddress, ROMCS);
	#endif

	/* output bits 23-25 of MMS Page 5 (Index 33H)
		also enables MMS Page 5 and sets the ROM CSn bits.
	*/
	OutpWord(0x22, 0x0833 +
		((WORD)_lshru(lAddress, (23-8)) & 0x0700) +
		(WORD)(ROMCS << 14));

	/* output bits 15-22 of MMS Page 5 (Index 32h) */
	OutpWord(0x22, 0x0032 + ((WORD)_lshru(lAddress, (15-8)) & 0xFF00));

	/* cannot fail */
	return TRUE;
}

/*
	$DOCPROC:init_page

	Initialize ROMCS(n) for 16 bits wide. This is required to access
	the 4 Meg of on-board Flash correctly. Sets up Configuration
	Register A for the particular ROM CS. (Register Index 23, 25, or
	27h).
*/
bool init_page(void)
{
	unsigned char oldVal;

	/* get the old value of Configuration Register A */
	OutpByte(0x22, 0x23 + (ROMCS*2));
	oldVal = InpByte(0x23);

	/* set for 16 bits wide, bits 2:1 of Conf. Register A */
	oldVal = (oldVal & 0xF9) | 0x04;

	/* set up the proper Index Register, depending on which ROM CS.
		Using out word, we can both set the index (I/O port 22H), and
		the data value (I/O port 23H) with one write. Word value for
		ROM CS1 would be 0425H.
	*/
	OutpWord(0x22, ((0x0023 + (ROMCS*2)) | (oldVal << 8)));

	/* worked */
	return TRUE;
}

/*
	$DOCPROC:uninit_page
	No action required for the SC400.
*/
void uninit_page(void)
{
}
