//  $Id: _capi.c 1.12 2003/05/15 17:11:32 johnb Exp $
//
//  *** DO NOT REMOVE THE FOLLOWING NOTICE ***
//
//  Copyright (c) 1996-2003 Datalight, Inc.
//  All Rights Reserved
//
//  Datalight, Inc. is a Washington corporation located at:
//
//  21520 30th Dr SE, Suite 110,          Tel: +1.425.951.8086
//  Bothell, WA  98021                    Fax: +1.425.951.8094
//  USA                                   Http://www.datalight.com
//
//  This software, including without limitation all source code and
//  documentation, is the confidential, trade secret property of
//  Datalight, Inc., and is protected under the copyright laws of
//  the United States and other jurisdictions.  Portions of the
//  software may also be subject to one or more the following US
//  patents: US#5860082, US#6260156.
//
//  In addition to civil penalties for infringement of copyright
//  under applicable U.S. law, 17 U.S.C. 1204 provides criminal
//  penalties for violation of (a) the restrictions on circumvention
//  of copyright protection systems found in 17 U.S.C. 1201 and
//  (b) the protections for the integrity of copyright management
//  information found in 17 U.S.C. 1202.
//
//  U.S. Government Restricted Rights.  Use, duplication,
//  reproduction, or transfer of this commercial product and
//  accompanying documentation is restricted in accordance with
//  FAR 12.212 and DFARS 227.7202 and by a License Agreement.
//
//  IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS
//  PROTECTED UNDER A SOURCE CODE AGREEMENT, NON-DISCLOSURE
//  AGREEMENT (NDA), OR SIMILAR BINDING CONTRACT BETWEEN DATALIGHT,
//  INC. AND THE LICENSEE ("BINDING AGREEMENT").  YOUR RIGHT, IF ANY,
//  TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,IS SUBJECT
//  TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT, AND BY
//  USING THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE
//  TO BE BOUND BY THE TERMS OF THE BINDING AGREEMENT.  CONTACT
//  DATALIGHT, INC. AT THE ADDRESS SET FORTH ABOVE IF YOU OBTAINED
//  THIS SOFTWARE IN ERROR.
//
/************************************************************************
	_CAPI.C

	Copyright (c) 1996-2001 Datalight, Inc.
	Confidential and Proprietary
	All Rights Reserved

	Provides the compiler specific portions of the CAPI module.

	History:
	07/19/2002 jmb IR#4032 corrected compilation errors with MSVC
	02/13/2002 jmb IR#3939
	               corrected variable declaration in FindSocketsVector
	               when compiling in C++ mode.
   08/22/2001 jmb added DJGPP support
   04/19/2001 jmb changed ISR_ROUTINE to _loadds
   03/20/2001 jmb Added FindSocketsVector to search the vector table
                  for the Sockets kernel.  We look for the string
                  "TCPTSR" to verify that Sockets is in memory.
                  I removed the code that checked the interrupt
                  vector for a non-zero value.
   03/16/2001 jmb added check for actual interrupt vector being set
                  changed error check on return from Sockets api
                  to check cflag rather than use flags which is not
                  available in MSVC.  I also modified the definition
                  of AsyncNotificationHandler to compile under either
                  SDTK or MSVC.
	1.01 2000-12-09 pkg Factored out non portable code
	1.00 2000-09-08 pkg Created
 ************************************************************************/

#include <dos.h>

#ifdef __DJGPP__
#include <dpmi.h>
#include <go32.h>
#include <conio.h>
#endif


#include "compiler.h"
#include "capi.h"

/*
	Prototypes
*/
int FindSocketsVector(void);

/*
	Stack size of stack for asynchronous callbacks
*/
#define HANDLER_STACK_SIZE 5000

/*
	The following global is used to tell the CallSocketsDosApi()
	function what interrupt the Sockets stack is hooking.  By
	default this is 0x61 unless overidden on the command line
	load of SOCKETM or SOCKETSP.
*/
int iSocketsApiInterrupt = 0x00;


/*
	Parameters
	px86Regs, pointer to an X86Regs structure which holds the
	registers to call the Sockets interrupt with.  This structure
	is updated with the new register values after the Sockets
	interrupt is made.
	Description
	Calls the Sockets kernel via a software interrupt described
	by iSocketsApiInterrupt with the registers defined in the
	X86Regs structure.
	Returns
	On success, ax value returned by the Sockets call.
	On error (carry flag set), -1 if the Sockets API is not found
*/
int CallSocketsDosApi(X86Regs * px86regs)
{
#ifndef __DJGPP__
	union REGS  r;
	struct SREGS s;
#else
   __dpmi_regs r;
#endif

	/* Clear the current error status */
	iNetErrNo = iNetSubErrNo = 0;

	/* Make certain the sockets interrupt is set */
	if (iSocketsApiInterrupt == 0)
	{
		iSocketsApiInterrupt = FindSocketsVector();
		if (iSocketsApiInterrupt == 0)
		{
			iNetErrNo = ERR_API_NOT_LOADED;
			return -1;
		}
	}

	/*
		Call the sockets interrupt with the appropriate
		registers setup for the sockets API
	*/
	r.x.ax = px86regs->ax;
	r.x.bx = px86regs->bx;
	r.x.cx = px86regs->cx;
	r.x.dx = px86regs->dx;
	r.x.si = px86regs->si;
	r.x.di = px86regs->di;

#ifdef __DJGPP__
	r.x.ds = px86regs->ds;
	r.x.es = px86regs->es;

   // clear stack registers so that DPMI provides the stack
   r.x.ss = 0;
   r.x.sp = 0;

   // call the dpmi handler
   __dpmi_int(iSocketsApiInterrupt, &r);
#else
	s.ds = px86regs->ds;
	s.es = px86regs->es;
	int86x(iSocketsApiInterrupt, &r, &r, &s);
#endif

	px86regs->ax = r.x.ax;
	px86regs->bx = r.x.bx;
	px86regs->cx = r.x.cx;
	px86regs->dx = r.x.dx;
	px86regs->si = r.x.si;
	px86regs->di = r.x.di;

#ifdef __DJGPP__
	px86regs->ds = r.x.ds;
	px86regs->es = r.x.es;
#else
	px86regs->ds = s.ds;
	px86regs->es = s.es;
#endif

	/*
		Any errors are reported with a carry flag set and the
		network error is in the low eight bits and any
		sub-error codes are in the upper eight bits
	*/
	if (CARRY_FLAG_CHECK)
	{
		iNetErrNo = px86regs->ax & 0x00FF;
		iNetSubErrNo = px86regs->ax >> 8;
		return -1;
	}
	else
	{
		return px86regs->ax;
	}
}


#ifdef DJGPP
extern __dpmi_regs cb_regs;
#endif

typedef int (D_FAR *FH)(int,int,DWORD);

int LOADDS D_FAR AsyncNotificationHandler(void)
{
#ifdef __DJGPP__
   __dpmi_regs r;
#endif

   static int iSocket, iEvent;
   static WORD wArgLo, wArgHi, wHintLo, wHintHi;

	static DWORD dwArg;
	static FH Handler;
#ifndef __DJGPP__
	static int iSaveSS, iSaveSP;
	static int iStackPointer;
#endif
	static int iSaveNetErrNo, iSaveNetSubErrNo;
	static char rgcStack[HANDLER_STACK_SIZE];

#ifndef __DJGPP__
	iStackPointer = (int)rgcStack + HANDLER_STACK_SIZE - 10;
#endif

#ifdef __DJGPP__

   iSocket = cb_regs.x.bx;
   iEvent  = cb_regs.x.cx;
   wArgLo  = cb_regs.x.dx;
   wArgHi  = cb_regs.x.si;
   wHintHi = cb_regs.x.es;
   wHintLo = cb_regs.x.di;

#else
	_ASM {

		// It is safe to use local variables as async notifications don't nest
		// We can safely ignore BP as long as we do not use any auto variables
		// Save arguments in registers into local variables
		mov     iSocket,bx;
		mov     iEvent,cx;
		mov     wArgLo,dx;
		mov     wArgHi,si;
		mov     wHintHi,es;
		mov     wHintLo,di;

		// Save stack registers
		mov     iSaveSS,ss;
		mov     iSaveSP,sp;

		// Switch stack to local stack
		mov     ax,ds;
		mov     ss,ax
		mov   sp,iStackPointer;
	}
#endif
	
	// Save the error variables just incase we interrupted while they have
	// been set, but not used yet and our user calls CAPI.

	iSaveNetErrNo = iNetErrNo;
	iSaveNetSubErrNo = iNetSubErrNo;

	((WORD *)&Handler)[0] = wHintLo;
	((WORD *)&Handler)[1] = wHintHi;
	((WORD *)&dwArg)[0] = wArgLo;
   ((WORD *)&dwArg)[1] = wArgHi;

   if (Handler)
#ifdef __DJGPP__
   {
      r.x.bx = iSocket;
      r.x.cx = iEvent;
      r.x.dx = wArgLo;
      r.x.si = wArgHi;
      r.x.es = wHintHi;
      r.x.di = wHintLo;

      r.x.ss = 0;
      r.x.sp = 0;

      r.x.cs = wHintHi;
      r.x.ip = wHintLo;

      _go32_dpmi_simulate_fcall(&r);
   }
#else
      (*Handler)(iSocket,iEvent, dwArg);

	_ASM 
   {
		// Restore stack registers
		mov     ss,iSaveSS;
		mov     sp,iSaveSP;
	}

#endif

	// resore error variables
	iNetErrNo = iSaveNetErrNo;
	iNetSubErrNo = iSaveNetSubErrNo;

	return 0;
}

/*
 *----------------------------------------------------------------------------
 * $DocProc: FindSocketsVector
 *
 * OnEntry
 *   Nothing
 *
 * OnExit
 *   returns SOCKETS vector number or 0 if not found.
 *
 * Modifies
 *   none
 *
 *---------------------------------------------------------------------------- 
 */
int FindSocketsVector()
{
#ifdef __DJGPP__
   __dpmi_raddr api_info;
   char cTempTSRID[10];
#else
#ifdef __cplusplus
	void (interrupt *piAPIInterruptVector)(...);
#else
	void (interrupt *piAPIInterruptVector)();
#endif
#endif
	static char cSockTSRID[] = "TCPTSR";
	int iAPIInterrupt, i;
	char D_FAR *pTSRID;

	for (iAPIInterrupt = 0x20; iAPIInterrupt < 0x100; iAPIInterrupt++) 
	{
#ifdef __DJGPP__
      __dpmi_get_real_mode_interrupt_vector(iAPIInterrupt, &api_info);
#else
		piAPIInterruptVector = _dos_getvect(iAPIInterrupt);
#endif

#ifdef __DJGPP__
		if (api_info.segment == 0 && api_info.offset16 == 0)
#else
		if (piAPIInterruptVector == 0)
#endif
			continue;

#ifdef __DJGPP__
      dosmemget((DWORD)(api_info.segment << 4) + api_info.offset16,
               10, cTempTSRID);
      pTSRID = &cTempTSRID[3];
#else
		pTSRID = (char far *)piAPIInterruptVector + 3;
#endif
		for (i = 0; i < 6; i++) 
		{
			if (pTSRID[i] != cSockTSRID[i])
				break;
		}
		if (i == 6)
			break;
	}
	if (iAPIInterrupt == 0x100) 
		iAPIInterrupt = 0;
	return iAPIInterrupt;
}
