//
//   $Id: ssicgi.c 1.3 2002/10/16 16:57:00 peterb Exp $
//
//   *** DO NOT REMOVE THE FOLLOWING NOTICE ***
//
//   Copyright (c) 1993 - 2002 Datalight, Inc.
//   All Rights Reserved
//
//   Datalight, Inc. is a Washington corporation with an address at
//   21520 30TH DR SE MS110 Bothell, WA, 98021
//   Tel: 800.221.6630  Fax: 425.951.8091
//   Http://www.datalight.com
//
//   This software, including without limitation all source code and
//   documentation, is the confidential, trade secret property of
//   Datalight, Inc., and is protected under the copyright laws of
//   the United States and other jurisdictions. Portions of the
//   software may also be subject to one or more the following US
//   patents: US#5860082, US#6260156.
//
//   In addition to civil penalties for infringement of copyright
//   under applicable U.S. law, 17 U.S.C. 1204 provides criminal
//   penalties for violation of (a) the restrictions on circumvention
//   of copyright protection systems found in 17 U.S.C. 1201 and
//   (b) the protections for the integrity of copyright management
//   information found in 17 U.S.C. 1202.
//
//   U.S. Government Restricted Rights. Use, duplication,
//   reproduction, or transfer of this commercial product and
//   accompanying documentation is restricted in accordance with
//   FAR 12.212 and DFARS 227.7202 and by a License Agreement.
//
//   IN ADDITION TO COPYRIGHT AND PATENT LAW, THIS SOFTWARE IS
//   PROTECTED UNDER A SOURCE CODE AGREEMENT, NON-DISCLOSURE
//   AGREEMENT (NDA), OR SIMILAR BINDING CONTRACT BETWEEN DATALIGHT,
//   INC. AND THE LICENSEE ("BINDING AGREEMENT"). YOUR RIGHT, IF ANY,
//   TO COPY, PUBLISH, MODIFY OR OTHERWISE USE THE SOFTWARE,IS SUBJECT
//   TO THE TERMS AND CONDITIONS OF THE BINDING AGREEMENT, AND BY
//   USING THE SOFTWARE IN ANY MANNER, IN WHOLE OR IN PART, YOU AGREE
//   TO BE BOUND BY THE TERMS OF THE BINDING AGREEMENT. CONTACT
//   DATALIGHT, INC. AT THE ADDRESS SET FORTH ABOVE IF YOU OBTAINED
//   THIS SOFTWARE IN ERROR.
//
/*
	$DOCFILE:SSICGI.C

	Copyright (C) 1999-2001 Datalight, Inc.
	All Rights Reserved

	Server Side Include (SSI) functions
	for Sockets HTTP server

	$DOCHISTORY:

	2001-05-11	GPvN cleanup
	2000-05-18	GVvN first version
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>
#include <io.h>
#include "cgiadap.h"
#include "ssi.h"

typedef struct tagSSIVAR
{
  char* pszName;
  char* pszValue;
  struct tagSSIVAR *pNext;
} SSIVAR, *PSSIVAR;



typedef struct
{
	SSIVAR *pSsivarFirst;
	FILE *pFile;

	//parsing variables
#define DATA_SIZE	256
#define TAG_SIZE	128
	char rgDatabuf[DATA_SIZE];//data buffer
	int iBufpos;		//data insertion or extraction point into buffer
	char rgTag[TAG_SIZE];	//the ssi tag that we are saving
	int iTagpos;		//position into tag we are saving
	char *pData;		//pointer to any sz - qued data
	char rgVABuf[7];	//buffer holding virtual array + 1 other byte
	int iVpos;			//pointer in virtual array ("<!--#")
} PARAMS;


int far Callback(HTTP_PARAMS far *pPars);

/*
initialize ssi adapter: hook HTTP server
*/
static char *pszCgiNameSave;
int CgiInit(char *pszCgiName)
{
	if (pszCgiNameSave)
		return ERR_DUPL;
	if ((pszCgiNameSave = malloc(strlen(pszCgiName) + 1)) == 0)
		return NO_SPACE;
	strcpy(pszCgiNameSave, pszCgiName);
	return HttpRegister(pszCgiName, Callback, F_OVERLAP, 0);
}

void CgiQuit(void)
{
	HttpDeRegister(pszCgiNameSave);
	if (pszCgiNameSave) {
		free(pszCgiNameSave);
		pszCgiNameSave = 0;
	}
}

static void FreeParams(PARAMS *pPars)
{
	while (pPars->pSsivarFirst) {
		SSIVAR *pSsi = pPars->pSsivarFirst;
		pPars->pSsivarFirst = pSsi->pNext;
		free(pSsi);
	}
	fclose(pPars->pFile);
	free(pPars);
}

//receive ssi string. Parse and find value.
//Format of string:(command parameter="argument"--) and should match exactly
static void DoSSI(PARAMS *pPars)
{
	char *pCommand = pPars->rgTag;
	char *pParameter;
	char *pArgument;
	char *pTerm;
	static char pszNOVAR[] = "BAD_SSI_VAR";

	//parse argument string
	pPars->pData = pszNOVAR;//safety
	if((pParameter = strchr(pCommand, ' ')) == 0)
		return;
	*pParameter++ = 0;
	if((pArgument = strchr(pParameter, '=')) == 0)
		return;
	*pArgument++ = 0;
	pArgument++;//skip '"'
	if((pTerm = strchr(pArgument, '"')) == 0)
		return;
	*pTerm = 0;

	//check type
	if (!stricmp(pCommand, "echo") && !stricmp(pParameter, "var")) {
		//walk through list and set
		SSIVAR *pVar = pPars->pSsivarFirst;
		while (pVar) {
			if (!strcmp(pVar->pszName, pArgument)) {
				pPars->pData = pVar->pszValue;
				break;
			}
			pVar = pVar->pNext;
		}
	}
}

int OutData(HTTP_PARAMS far *pHTPars)
{
	PARAMS *pPars = (PARAMS *)pHTPars->dwSessionID;	//parameter block
	int iTCount = pHTPars->iValue;						//transmit count
	int iSend;
	static char rgV[] = "<!--#";
	int iChar = 0;
	char c;

	while (iTCount > 0) {

		//fill block (calculate)
		while (pPars->iBufpos < DATA_SIZE) {
			if (pPars->pData) {
				c = *pPars->pData++;
				if (!*pPars->pData)	//0 - terminator
					pPars->pData = 0;
			}
			else {
				iChar = getc(pPars->pFile);
				if (iChar == EOF)
					break;
				c = iChar & 0xff;
			}
			if (pPars->iVpos) {	//if in virtual buffer
				if (rgV[pPars->iVpos] == c)
					pPars->iVpos++;
				else {
					//que data from virtual array and this byte
					memcpy (pPars->rgVABuf, rgV, pPars->iVpos);
					pPars->rgVABuf[pPars->iVpos++] = c;
					pPars->rgVABuf[pPars->iVpos] = 0;	//terminate
					pPars->iVpos = 0;
					pPars->pData = pPars->rgVABuf;
				}
				if (pPars->iVpos == 5) {
					//this is SSI string
					pPars->iVpos = 0;
					pPars->iTagpos = -1;
				}
			}
			else if (pPars->iTagpos) {	//in ssi tag
				if (pPars->iTagpos == -1)
					pPars->iTagpos = 0;
				if (c == '>') {
					DoSSI(pPars);	//will set pPars->pData
					pPars->iTagpos = 0;
				}
				else if (pPars->iTagpos < TAG_SIZE)
					pPars->rgTag[pPars->iTagpos++] = c;
			}//else normal
			else if (c == '<' && !pPars->pData)
				pPars->iVpos = 1;
			else {
				pPars->rgDatabuf[pPars->iBufpos++] = c;
				if (--iTCount <= 0)
					break;
			}
		}

		//transmit calculated block
		if((iSend = HttpSendData(pHTPars->iHandle,
		 pPars->rgDatabuf, pPars->iBufpos)) != pPars->iBufpos)
		{
			//couldnt send all
			if (iSend <= 0) {	//error
				if (iSend != WOULDBLK) {//serious error
					FreeParams(pPars);
					return RET_ERR;
				}
				iSend = 0;
			}
			//reque data
			memcpy (pPars->rgDatabuf, pPars->rgDatabuf + iSend,
			 pPars->iBufpos - iSend);
		}
		else if (feof(pPars->pFile)) {
			FreeParams(pPars);
			return RET_DONE;
		}
		pPars->iBufpos -= iSend;
	}
	return RET_OK;
}


/*
 * callback
 */
int far Callback(HTTP_PARAMS far *pPars)
{

	PARAMS *pSsipars;
	char rgQuery[128];
	int iLen;

	switch (pPars->iReason) {
	case R_NEWREQ:
		//pfFile
		if ((pSsipars = calloc(1, sizeof(PARAMS))) == 0)
			return RET_ERR;
		iLen = _fstrcpy(rgQuery, pPars->szQuery) - rgQuery;
		HttpGetData(pPars->iHandle, rgQuery + iLen, 128 - iLen);
		SetVariables(rgQuery, (int)pSsipars);
		pPars->dwSessionID = (long)pSsipars;
		break;
	case R_OUTDATA:
		return OutData(pPars);
	case R_ENDDATA:
	case R_CLOSED:
		//were supposed to close ourselves. This must indicate error state
		FreeParams((PARAMS *)pPars->dwSessionID);
		return RET_ERR;

	}
	return RET_OK;
}

int SetFile(int iHandle, char *pszFilename)
{
	PARAMS *pParams = (PARAMS *)iHandle;

	if ((pParams->pFile = fopen(pszFilename, "rb")) == 0)
		return BADFILENAME;
	return OK;
}

int AddVar(int iHandle, char *pszName, char *pszValue)
{
	PARAMS *pParams = (PARAMS *)iHandle;
	int iSizePars = sizeof(SSIVAR);
	int iSizeName = strlen(pszName) + 1;
	int iSizeValue = strlen(pszValue) + 1;
	SSIVAR *pVar = calloc(1, iSizePars + iSizeName + iSizeValue);
	if (!pVar)
		return -1;
	pVar->pNext = pParams->pSsivarFirst;
	pParams->pSsivarFirst = pVar;
	pVar->pszName = (char *)pVar + iSizePars;
	pVar->pszValue = pVar->pszName + iSizeName;
	strcpy (pVar->pszName, pszName);
	strcpy (pVar->pszValue, pszValue);
	return 0;
}
